@@ -1,10 +1,11 @@
-/* SPDX-FileCopyrightText: 2021 John Scott <jscott@posteo.net>
+/* SPDX-FileCopyrightText: 2021-2022 John Scott <jscott@posteo.net>
  * SPDX-License-Identifier: GPL-2.0-or-later */
 #include <assert.h>
 #include <flint/fmpz.h>
 #include <flint/fmpq.h>
 #include <flint/arith.h>
 #include <flint/fmpq_mat.h>
+#include <flint/fmpq_vec.h>
 
 #define LIMIT 10
 
@@ -29,7 +30,6 @@
 int main(void) {
 	fmpq_mat_t binomcoeffs;
 	fmpq_mat_init(binomcoeffs, LIMIT, LIMIT);
-
 	fmpq_mat_zero(binomcoeffs);
 	for(slong row = 0; row < LIMIT; row++) {
 		for(slong col = 0; col <= row; col++) {
@@ -40,27 +40,16 @@ int main(void) {
 	}
 	assert(fmpq_mat_is_integral(binomcoeffs));
 
-	fmpq_mat_t bernoullis;
-	fmpq_mat_init(bernoullis, LIMIT, LIMIT);
-	int s = fmpq_mat_inv(bernoullis, binomcoeffs);
-	assert(s);
+	fmpq *const bernoullis = _fmpq_vec_init(LIMIT + 1);
+	arith_bernoulli_number_vec(bernoullis, LIMIT + 1);
+
+	fmpq *const out = _fmpq_vec_init(LIMIT);
+	fmpq_mat_mul_fmpq_vec(out, binomcoeffs, bernoullis + 1 /* skip B_0 */, LIMIT);
+	for(slong i = 0; i < LIMIT; i++) {
+		assert(fmpq_is_pm1(out + i));
+	}
+
+	_fmpq_vec_clear(out, LIMIT);
+	_fmpq_vec_clear(bernoullis, LIMIT + 1);
 	fmpq_mat_clear(binomcoeffs);
-
-	fmpq_mat_t negones;
-	fmpq_mat_init(negones, LIMIT, 1);
-	for(slong i = 0; i < LIMIT; i++) {
-		*fmpq_mat_entry_num(negones, i, 0) = WORD(-1);
-		*fmpq_mat_entry_den(negones, i, 0) = WORD(1);
-	}
-	assert(fmpq_mat_is_integral(negones));
-
-	fmpq_mat_mul(bernoullis, bernoullis, negones);
-	fmpq_mat_clear(negones);
-
-	fmpq exactbern;
-	for(slong i = 0; i < LIMIT; i++) {
-		arith_bernoulli_number(&exactbern, i + 1);
-		assert(fmpq_equal(&exactbern, fmpq_mat_entry(bernoullis, i, 0)));
-	}
-	fmpq_mat_clear(bernoullis);
 }
